/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <mysql.h>
#include <ndb_init.h>
#include <mgmapi.h>

#include "config.h"

extern "C" int load_defaults(const char *conf_file, const char **groups,int *argc, char ***argv);

int main(int argc, char **argv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 0;
	int wait         = 2;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, &argc, &argv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
        {"wait"          , 1, 0, 'w'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:w:t:";
	int opt, option_index;
	char *waitstr = "completed";

	opterr = 0;
	do {
		opt = getopt_long(argc, argv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
		    connectstr = optarg;
		    break;
		case 't':
		    no_retries = atoi(optarg);
		    break;
		case 'w':
			wait = atoi(optarg);
			switch (wait) {
			case 0:
			  waitstr = "requested";
			  break;
			case 1:
			  waitstr = "started";
			  break;
			case 2:
			  waitstr = "completed";
			  break;
			default:
			  fprintf(stderr, "invalid wait code %d, only 0-2 are allowed\n", wait);
			  exit(EXIT_FAILURE);
			  break;
			}
            break;
		default:
			break;
		}
	} while ( opt != -1);

	NdbMgmHandle h = ndb_mgm_create_handle();

    ndb_mgm_set_connectstring(h, connectstr);

    struct ndb_mgm_reply reply;
	unsigned int id;

    if (-1 != ndb_mgm_connect(h, no_retries, retry_delay, verbose)) {
	  printf("starting backup\n");
	  if (-1 ==ndb_mgm_start_backup(h, wait, &id, &reply)) {
		fprintf(stderr, "backup failed, error %d: %s\n", reply.return_code, reply.message);
		exit(EXIT_FAILURE);
      }
	  
	  if (wait == 0) {
		id = 0;
	  }
	  printf("backup %d %s\n", id, waitstr);
    } else {
		fprintf(stderr, "can't connect to 'connectstr'\n");
		exit(EXIT_FAILURE);
	}

	ndb_end(0);

	exit(EXIT_SUCCESS);
}


